#!/bin/bash
#
# xfce-svn-duilder.sh - small utility to update & build Xfce modules
#
# Version: 1.1
# Author: Mateusz Loskot <mateusz at loskot dot net>
# Update: http://mateusz.loskot.net
#
# Freely redistributable and modifiable. Use at your own risk.  
#
#####################################################################
# Config Section
VERBOSE=1
CONFIGURE_OPTIONS="--prefix=/usr/local"
# Commands
CHECKOUT="svn checkout"
UPDATE="svn update"
CONFIGURE="./autogen.sh"
MAKE="make"
INSTALL="sudo make install"
CLEAN="make clean"
# Files
MAKEFILE="./Makefile"
LOG="`pwd`/build.log"
EXTRALOG="`pwd`/configure.log"
# End of Config Section
#####################################################################

# Local configuration variables
RUN_CHECKOUT=0
RUN_UPDATE=0
RUN_CONFIGURE=0
RUN_MAKE=0
RUN_INSTALL=0
RUN_CLEAN=0
RUN_ALL=0
MODULES_LIST="" # modules database file

usage()
{
	echo "Usage xfce-svn-builder.sh xfce-svn-modules.lst [OPTIONS]"
	echo "  --help                 prints this message"
	echo "  --all                  run all targets in following order: update, make, install, clean (default)"
	echo "  --checkout-only        checkout clean copy of every module to separate sandbox directory"
	echo "  --update-only          update every module against SVN repository"
	echo "  --build-only           run 'make' for every module"
	echo "  --install-only         run only 'make install' for every module"
	echo "  --clean-only           run only 'make clean' for every module"
}

check_env()
{
	local TARGET=""
		
	# Check required tools	

	TARGET=`which make`
	if [[ -z  ${TARGET} ]]; then
		echo "Can not find 'make' in the path!"
		exit
	fi
	
	TARGET=`which svn`
	if [[ -z  ${TARGET} ]]; then
		echo "Can not find 'svn' in the path!"
		exit
	fi
}

read_modules_db()
{
	MODULES_DB=$1
		
	if [ -z $MODULES_DB ]; then
		usage
	fi

	if [ ! -f $MODULES_DB ]; then
		echo "[ERROR] $MODULES_DB does not exist!"
		exit 1			
	fi
	
	# Read modules list
	lineno=0
	while read line;
	do
		# Suppress comments
		if [ "${line:0:1}" != "#" ]; then	
			MODULES_LIST[${lineno}]="${line}"
			(( lineno++ ))
		fi
	done < ${MODULES_DB}

	# Test
	#count=${lineno}
	#for (( lineno=0; lineno<=count; lineno++ )); do
	#		echo ${MODULES_LIST[${lineno}]}
	#	done
}

progress()
{
	if [ $VERBOSE = 1 ]; then
		echo -n "..$1.."
	else
		echo -n "."
	fi
}

log_module_start()
{
	module=$( printf "%s\n" "$1" | tr '[a-z]' '[A-Z]'| tr -d '/')
	echo "###### $module ############################" >> $LOG
}

log_module_end()
{
	echo "$1" >> $LOG
}

log_module_message()
{
	echo "$1" >> $LOG
}

build_module()
{
	local MODULE=$1

	if [ -z $1 ]; then
		echo "[ERROR] Module directory expected!"
		exit 1
	fi

	log_module_start $1

	# Enter existing module sandbox dir
	if [[ ${RUN_CHECKOUT} -eq 0 ]]; then
		if [[ -d ${MODULE} ]]; then
			cd $1
		else
			echo "'${MODULE}' module not found!"
			exit 1
		fi
	fi

	################################
	# Checkout source tree with SVN repo
	if [[ ${RUN_CHECKOUT} -eq 1 ]]; then
		
		progress "checkout"
		
		local URL="http://svn.xfce.org/svn/xfce/${MODULE}/trunk"
		local CHECKOUT_CMD="$CHECKOUT $URL $MODULE"

		${CHECKOUT_CMD} > /dev/null 2>&1
		#local CHECKOUT_STATUS=$($CHECKOUT_CMD) 
		if [ $? != 0 ]; then
			log_module_message "[ERROR] $CHECKOUT failed"
			#log_module_message "---------- svn checkout output - begin ----------"
			#log_module_message "${CHECKOUT_STATUS}"
			#log_module_message "---------- svn checkout output - end ----------"
			exit 1
		fi

		log_module_message "$CHECKOUT - OK"
	
	elif [[ ${RUN_UPDATE} -eq 1 ]]; then   # Update source tree with SVN repo
		progress "update"
			
		#$UPDATE > /dev/null 2>&1 # Deprecated
		local UPDATE_STATUS=$($UPDATE)
		if [ $? != 0 ]; then
			log_module_message "[ERROR] $UPDATE failed"
			exit 1
		fi
	
		log_module_message "---------- svn update output - begin ----------"
		log_module_message "$UPDATE_STATUS"
		log_module_message "---------- svn update output - end ----------"
		log_module_message "$UPDATE - OK"
	fi

	################################
	# Configure
	if [[ ${RUN_CONFIGURE} -eq 1 ]]; then 
		progress "configure"
			
		if [ -x ${CONFIGURE} ]; then
			if [ -a $MAKEFILE ]; then
				progress "clean"
				$CLEAN > /dev/null 2>&1
				if [ $? != 0 ]; then
					log_module_message "Intermediate $CLEAN - OK"
				fi
			fi
			${CONFIGURE} ${CONFIGURE_OPTIONS} >> $EXTRALOG
		fi

		if [ $? != 0 ]; then
			log_module_message "[ERROR] $CONFIGURE script failed"
			exit 1
		fi

		log_module_message "$CONFIGURE script - OK"
	fi

	################################
	# Build
	if [[ ${RUN_MAKE} -eq 1 ]]; then 
		progress "make"
			
		$MAKE > /dev/null 2>&1
		if [ $? != 0 ]; then
			ERRMSG="[ERROR] $MAKE command failed"
			echo "$ERRMSG"
			log_module_message "$ERRMSG"
			exit 1
		fi

		log_module_message "$MAKE - OK"
	fi
	
	################################
	# Install
	if [[ ${RUN_INSTALL} -eq 1 ]]; then 
		progress "install"
			
		$INSTALL > /dev/null 2>&1
		if [ $? != 0 ]; then
			ERRMSG="[ERROR] $INSTALL command failed"
			echo "$ERRMSG"
			log_module_message "$ERRMSG"
			exit 1
		fi

		log_module_message "$INSTALL - OK"
	fi
	
	################################
	# Clean only after installation
	if [[ ${RUN_CLEAN} -eq 1 ]]; then 
		progress "clean"
			
		$CLEAN > /dev/null 2>&1
		if [ $? = 0 ]; then
			log_module_message "$CLEAN - OK"
		fi
	fi

	finish_module
}

finish_module()
{
	log_module_end "SUCCESS!"
	
	# Go back to the modules root dir
	if [[ ${RUN_CHECKOUT} -eq 0 ]]; then
		cd ..
	fi
}

build_all_modules()
{
	for MODULE in  ${MODULES_LIST[@]};
	do
		if [[ ${RUN_CHECKOUT} -eq 0 ]] && [[ ! -d ${MODULE} ]]; then
			echo "'${MODULE}' module not found!"
			exit 1
		fi
		
		echo -n "${MODULE}."
		build_module ${MODULE}
		if [ $? != 0 ]; then
			exit
		else
			echo "...OK"
		fi
	done

	echo "Done!"
}

########################################
# Script entry point

# Check tools
check_env

# Parse command line parameters

if [[ ! ${#} -lt 3 ]]; then
	usage
	exit
fi

if [[ ! -z ${1} ]]; then
	if [[ "${1}" = "--help" ]]; then
		usage
		exit
	elif [[ ! -f ${1} ]]; then
		echo "Modules database '${1}' not found!"
		exit 1
	fi
fi

if [[ ! -z ${1} ]] && [[ ${#} -eq 1 ]]; then
	(( RUN_ALL=1 ))
elif [[ ${#} -eq 2 ]] && [[ ! -z ${2} ]]; then
	if [[ "${2}" = "--checkout-only" ]]; then
		(( RUN_CHECKOUT=1 ))
		(( RUN_UPDATE=0 ))
	elif [[ "${2}" = "--update-only" ]]; then
		(( RUN_UPDATE=1 ))
		(( RUN_CHECKOUT=0 ))
	elif [[ "${2}" = "--config-only" ]]; then
		(( RUN_CONFIGURE=1 ))
	elif [[ "${2}" = "--build-only" ]]; then
		(( RUN_MAKE=1 ))
		(( RUN_CONFIGURE=1 ))
	elif [[ "${2}" = "--install-only" ]]; then
		(( RUN_INSTALL=1 ))
	elif [[ "${2}" = "--clean-only" ]]; then
		(( RUN_CLEAN=1 ))
	elif [[ "${2}" = "--all" ]]; then
		(( RUN_ALL=1 ))
	else
		usage
		exit 1
	fi
else
	usage
	exit 1
fi

if [[ ${RUN_ALL} -eq 1 ]]; then
	# TODO: which one should be default: co or up?
	(( RUN_CHECKOUT=0 )) # Turned off by default
	(( RUN_UPDATE=1 ))
	(( RUN_CONFIGURE=1 ))
	(( RUN_MAKE=1 ))
	(( RUN_INSTALL=1 ))
	(( RUN_CLEAN=1 ))
fi

# Run builder
read_modules_db $1

cat /dev/null > $LOG
echo "Build log will be saved to $LOG file"

build_all_modules

# EOF

